@@ -2,7 +2,7 @@
 ### Multivariate kernel density derivative estimate 
 ###############################################################################
 
-kdde <- function(x, H, h, deriv.order=0, gridsize, gridtype, xmin, xmax, supp=3.7, eval.points, binned=FALSE, bgridsize, positive=FALSE, adj.positive, w, deriv.vec=TRUE)
+kdde <- function(x, H, h, deriv.order=0, gridsize, gridtype, xmin, xmax, supp=3.7, eval.points, binned=FALSE, bgridsize, positive=FALSE, adj.positive, w, deriv.vec=TRUE, verbose=FALSE)
 {
   r <- deriv.order
 
@@ -30,11 +30,10 @@ kdde <- function(x, H, h, deriv.order=0, gridsize, gridtype, xmin, xmax, supp=3.
   ## compute binned estimator
   if (binned)
   {
-    if (!missing(eval.points))
-      stop("Both binned=TRUE and eval.points are non-empty.")
-    
+    if (!missing(eval.points)) stop("Both binned=TRUE and eval.points are non-empty.")
     if (missing(bgridsize)) bgridsize <- default.gridsize(d)
-  
+    ##if (!identical(diag(diag(H)), H) & d > 1) stop("Binned estimation defined for diagonal H only")
+    
     if (positive & is.vector(x))
     {
       y <- log(x)
@@ -44,7 +43,7 @@ kdde <- function(x, H, h, deriv.order=0, gridsize, gridtype, xmin, xmax, supp=3.
       fhat$x <- x
     }
     else
-      fhat <- kdde.binned(x=x, H=H, h=h, deriv.order=r, bgridsize=bgridsize, xmin=xmin, xmax=xmax, w=w, deriv.vec=deriv.vec)
+      fhat <- kdde.binned(x=x, H=H, h=h, deriv.order=r, bgridsize=bgridsize, xmin=xmin, xmax=xmax, w=w, deriv.vec=deriv.vec, verbose=verbose)
   }
   else
   {
@@ -72,7 +71,7 @@ kdde <- function(x, H, h, deriv.order=0, gridsize, gridtype, xmin, xmax, supp=3.
       
       if (missing(eval.points))
       {
-        if (d==2) ##stop("Not yet implemented for 2 dimensions")
+        if (d==2) 
           fhat <- kdde.grid.2d(x=x, H=H, gridsize=gridsize, supp=supp, xmin=xmin, xmax=xmax, gridtype=gridtype, w=w, deriv.order=r, deriv.vec=deriv.vec)
         else if (d == 3) stop("Not yet implemented for 3 dimensions")
           ##fhat <- kde.grid.3d(x=x, H=H, gridsize=gridsize, supp=supp, xmin=xmin, xmax=xmax, gridtype=gridtype, w=w) 
@@ -114,12 +113,12 @@ kdde <- function(x, H, h, deriv.order=0, gridsize, gridtype, xmin, xmax, supp=3.
 ## Multivariate binned kernel density derivative estimate
 ###############################################################################
 
-kdde.binned <- function(x, H, h, deriv.order, bgridsize, xmin, xmax, bin.par, w, deriv.vec=TRUE , deriv.index)
+kdde.binned <- function(x, H, h, deriv.order, bgridsize, xmin, xmax, bin.par, w, deriv.vec=TRUE, deriv.index, Sdr.mat, verbose=FALSE)
 {
   r <- deriv.order
-  if(length(r)>1) stop("deriv.order should be a non-negative integer.")
+  if (length(r)>1) stop("deriv.order should be a non-negative integer.")
+  
   ## linear binning
-
   if (missing(bin.par))
   {
     if (is.vector(x)) {d <- 1; n <- length(w)}
@@ -128,14 +127,13 @@ kdde.binned <- function(x, H, h, deriv.order, bgridsize, xmin, xmax, bin.par, w,
     if (missing(w)) w <- rep(1,n)
     
     if (d==1)
-      if (missing(H)) H <- as.matrix(h^2)
+      if (missing(H)) { H <- as.matrix(h^2)} 
       else {h <- sqrt(H); H <- as.matrix(H)}
 
-    if (!identical(diag(diag(H)), H) & d > 1)
-      stop("Binned estimation defined for diagonal Sigma only")
-  
+    if (d==1) Hd <- H else Hd <- diag(diag(H))
     if (missing(bgridsize)) bgridsize <- default.gridsize(d)
-    bin.par <- binning(x=x, H=H, h, bgridsize, xmin, xmax, supp=3.7+max(r), w=w)
+    
+    bin.par <- binning(x=x, H=Hd, h=h, bgridsize, xmin, xmax, supp=3.7+max(r), w=w)
   }
   else
   {
@@ -148,69 +146,110 @@ kdde.binned <- function(x, H, h, deriv.order, bgridsize, xmin, xmax, bin.par, w,
       else {h <- sqrt(H); H <- as.matrix(H)}
   }
 
-
+ 
   if (d==1)
-    range.x <- list(range(bin.par$eval.points))
-  else
-    range.x <- lapply(bin.par$eval.points, range)
-
-  if (d==1)
-  {  
-    fhat.grid <- drvkde(x=bin.par$counts, drv=r, bandwidth=sqrt(diag(H)), binned=TRUE, range.x=range.x, se=FALSE, gridsize=bgridsize)
-    eval.points <- fhat.grid$x.grid
-    fhat.grid <- fhat.grid$est
+  {
+    fhat <- kdde.binned.1d(h=h, deriv.order=r, bin.par=bin.par)
+    eval.points <- fhat$eval.points
+    est <- fhat$estimate
   }
   else
   {
     ind.mat <- dmvnorm.deriv(x=rep(0,d), mu=rep(0,d), Sigma=H, deriv.order=r, only.index=TRUE, deriv.vec=TRUE)
-    ind.mat.unique <- unique(ind.mat)
-
-    fhat.grid <- list()
-    if (r >0)
-    {
-      if (missing(deriv.index))
-      {
-        for (r2 in 1:nrow(ind.mat.unique))
-        {
-          fhat.temp <- drvkde(x=bin.par$counts, drv=ind.mat.unique[r2,], bandwidth=sqrt(diag(H)), binned=TRUE, range.x=range.x, se=FALSE, gridsize=bgridsize)
-          map.index <- which.mat(ind.mat.unique[r2,], ind.mat)
-          if (deriv.vec)
-            for (m in 1:length(map.index)) fhat.grid[[map.index[m]]] <- fhat.temp$est
-          else
-            fhat.grid[[r2]] <- fhat.temp$est
-        }
-        eval.points <- fhat.temp$x.grid
-        if (!deriv.vec) ind.mat <- ind.mat.unique
-      }
-      else
-      {
-        fhat.grid[[1]] <- drvkde(x=bin.par$counts, drv=ind.mat[deriv.index,], bandwidth=sqrt(diag(H)), binned=TRUE, range.x=range.x, se=FALSE, gridsize=bgridsize)
-        eval.points <- fhat.grid[[1]]$x.grid
-        fhat.grid[[1]] <- fhat.grid[[1]]$est
-        ind.mat <- ind.mat[deriv.index,]
-      }
-    }
-    else
-    {
-      fhat.grid[[1]] <- drvkde(x=bin.par$counts, drv=ind.mat[1], bandwidth=sqrt(diag(H)), binned=TRUE, range.x=range.x, se=FALSE, gridsize=bgridsize)
-      eval.points <- fhat.grid[[1]]$x.grid
-      fhat.grid[[1]] <- fhat.grid[[1]]$est
-    }
+    fhat.grid <- kdde.binned.nd(H=H, deriv.order=r, bin.par=bin.par, Sdr.mat=Sdr.mat, verbose=verbose)
   }
 
   if (missing(x)) x <- NULL
   
   if (d==1)
-    fhat <- list(x=x, eval.points=unlist(eval.points), estimate=fhat.grid, h=h, H=h^2, gridtype="linear", gridded=TRUE, binned=TRUE, names=NULL, w=w, deriv.order=r, deriv.ind=r)
+    fhat <- list(x=x, eval.points=unlist(eval.points), estimate=est, h=h, H=h^2, gridtype="linear", gridded=TRUE, binned=TRUE, names=NULL, w=w, deriv.order=r, deriv.ind=r)
   else
-    fhat <- list(x=x, eval.points=eval.points, estimate=fhat.grid, H=H, gridtype="linear", gridded=TRUE, binned=TRUE, names=NULL, w=w, deriv.order=r, deriv.ind=ind.mat)
-    
+  {
+    if (r==0)
+      fhat <- list(x=x, eval.points=fhat.grid$eval.points, estimate=fhat.grid$estimate[[1]], H=H, gridtype="linear", gridded=TRUE, binned=TRUE, names=NULL, w=w, deriv.order=r, deriv.ind=ind.mat)
+    else
+      fhat <- list(x=x, eval.points=fhat.grid$eval.points, estimate=fhat.grid$estimate, H=H, gridtype="linear", gridded=TRUE, binned=TRUE, names=NULL, w=w, deriv.order=r, deriv.ind=ind.mat)
+  }
+  
   class(fhat) <- "kdde"
   
   return(fhat)
 }
 
+kdde.binned.1d <- function(h, deriv.order, bin.par)
+{
+  r <- deriv.order
+  n <- sum(bin.par$counts)
+  a <- min(bin.par$eval.points)
+  b <- max(bin.par$eval.points)
+  M <- length(bin.par$eval.points)
+  L <- min(ceiling((4+r)*h*(M-1)/(b-a)), M-1)
+  Keval <- dnorm.deriv(x=(b-a)*(0:L)/(M-1), mu=0, sigma=h, deriv.order=r)/n
+  est <- symconv.ks(Keval, bin.par$counts, skewflag=(-1)^r)
 
+  return(list(eval.points=bin.par$eval.points, estimate=est))
+}
+
+kdde.binned.nd <- function(H, deriv.order, bin.par, Sdr.mat, verbose=FALSE)
+{
+  d <- ncol(H)
+  r <- deriv.order
+  n <- sum(bin.par$counts)
+  a <- sapply(bin.par$eval.points, min)
+  b <- sapply(bin.par$eval.points, max)
+  M <- sapply(bin.par$eval.points, length)
+  L <- pmin(ceiling((4+r)*max(sqrt(abs(diag(H))))*(M-1)/(b-a)), M-1)
+
+  if (missing(Sdr.mat)) Sdr.mat <- Sdr(d=d, r=r)
+  if (d==2) xgrid <- expand.grid((b[1]-a[1])*(0:L[1])/M[1], (b[2]-a[2])*(0:L[2])/M[2])
+  if (d==3) xgrid <- expand.grid((b[1]-a[1])*(0:L[1])/M[1], (b[2]-a[2])*(0:L[2])/M[2], (b[3]-a[3])*(0:L[3])/M[3])
+  if (d==4) xgrid <- expand.grid((b[1]-a[1])*(0:L[1])/M[1], (b[2]-a[2])*(0:L[2])/M[2], (b[3]-a[3])*(0:L[3])/M[3], (b[4]-a[4])*(0:L[4])/M[4])
+  ##nxgrid <- nrow(xgrid)
+
+  deriv.index <- dmvnorm.deriv(x=rep(0,d), mu=rep(0,d), Sigma=H, deriv.order=r, add.index=TRUE, Sdr.mat=Sdr.mat, only.index=TRUE) ##Keval$deriv.ind
+  deriv.index.minimal <- dmvnorm.deriv(x=rep(0,d), mu=rep(0,d), Sigma=H, deriv.order=r, add.index=TRUE, Sdr.mat=Sdr.mat, only.index=TRUE, deriv.vec=FALSE)
+
+  Keval <- dmvnorm.deriv(x=xgrid, mu=rep(0,d), Sigma=H, deriv.order=r, add.index=TRUE, Sdr.mat=Sdr.mat, deriv.vec=FALSE)
+  Keval <- Keval$deriv/n
+  if (r==0) Keval <- as.matrix(Keval, ncol=1)
+  
+  est <- list()
+  if (verbose) pb <- txtProgressBar() 
+
+  ## loop over only unique partial derivative indices
+  nderiv <- nrow(deriv.index.minimal)
+  if (!(is.null(nderiv)))
+    for (s in 1:nderiv)
+    {
+      deriv.rep.index <- which.mat(deriv.index.minimal[s,], deriv.index)
+      Kevals <- array(Keval[,s], dim=L+1)
+      if (r==0) sf <- rep(1,d)
+      else sf <- (-1)^deriv.index.minimal[s,]
+      if (d==2) est.temp <- symconv2D.ks(Kevals, bin.par$counts, skewflag=sf)
+      if (d==3) est.temp <- symconv3D.ks(Kevals, bin.par$counts, skewflag=sf)
+      if (d==4) est.temp <- symconv4D.ks(Kevals, bin.par$counts, skewflag=sf)
+      for (s2 in 1:length(deriv.rep.index)) est[[deriv.rep.index[s2]]] <-  zapsmall(est.temp)
+      if (verbose) setTxtProgressBar(pb, s/nderiv)
+    }
+  else
+  {
+    for (s in 1:ncol(Keval))
+    {  
+      Kevals <- array(Keval[,s], dim=L+1)
+      if (r==0) sf <- rep(1,d)
+      else sf <- (-1)^deriv.index[s,]
+      if (d==2) est[[s]] <- symconv2D.ks(Kevals, bin.par$counts, skewflag=sf)
+      if (d==3) est[[s]] <- symconv3D.ks(Kevals, bin.par$counts, skewflag=sf)
+      if (d==4) est[[s]] <- symconv4D.ks(Kevals, bin.par$counts, skewflag=sf)
+      est[[s]] <- zapsmall(est[[s]])
+      if (verbose) setTxtProgressBar(pb, s/ncol(Keval))
+    }
+  }
+  if (verbose) close(pb)
+  
+
+  return(list(eval.points=bin.par$eval.points, estimate=est, deriv.order=r))
+}
 
 
 ##############################################################################################
@@ -294,7 +333,7 @@ kdde.grid.2d <- function(x, H, gridsize, supp, gridx=NULL, grid.pts=NULL, xmin,
       eval.pts <- permute(list(eval.x, eval.y))
 
       ## Create list of matrices for different partial derivatives
-      fhat <- dmvnorm.deriv(x=eval.pts, mu=x[i,], Sigma=H, deriv.order=r, Sdr=S2r)
+      fhat <- dmvnorm.deriv(x=eval.pts, mu=x[i,], Sigma=H, deriv.order=r, Sdr.mat=S2r)
       
       ## place vector of density estimate values `fhat' onto grid 'fhat.grid'
       for (k in 1:nderiv)
@@ -305,7 +344,7 @@ kdde.grid.2d <- function(x, H, gridsize, supp, gridx=NULL, grid.pts=NULL, xmin,
     for (k in 1:nderiv) fhat.grid[[k]] <- fhat.grid[[k]]/n
     gridx1 <- list(gridx[[1]], gridx[[2]]) 
 
-    ind.mat <- dmvnorm.deriv(x=rep(0,d), mu=rep(0,d), Sigma=H, deriv.order=r, Sdr=S2r, only.index=TRUE)
+    ind.mat <- dmvnorm.deriv(x=rep(0,d), mu=rep(0,d), Sigma=H, deriv.order=r, Sdr.mat=S2r, only.index=TRUE)
 
     if (!deriv.vec)
     {
@@ -333,21 +372,6 @@ kdde.grid.2d <- function(x, H, gridsize, supp, gridx=NULL, grid.pts=NULL, xmin,
 ## evaluated at each sample point
 #################################################################################################
 
-kdde.points <- function(x, H, eval.points, w, deriv.order=0, deriv.vec=TRUE) 
-{
-  n <- nrow(x)
-  Hs <- numeric(0)
-  for (i in 1:n)
-    Hs <- rbind(Hs, H)
-  r <- deriv.order
-  d <- ncol(H)
-  fhat <- dmvnorm.deriv.mixt(x=eval.points, mus=x, Sigmas=Hs, props=w/n, deriv.order=r, deriv.vec=deriv.vec, add.index=TRUE)
-
-  ##ind.mat <- dmvnorm.deriv(x=rep(0,d), mu=rep(0,d), Sigma=H, deriv.order=r, Sdr=Sdr(d=d, r=r), only.index=TRUE)
-  
-  return(list(x=x, eval.points=eval.points, estimate=fhat$deriv, H=H, gridded=FALSE, binned=FALSE, names=NULL, w=w, deriv.order=r, deriv.ind=fhat$deriv.ind))
-}
-
 kdde.points.1d <- function(x, h, eval.points, w, deriv.order=0) 
 {
   r <- deriv.order
@@ -357,12 +381,85 @@ kdde.points.1d <- function(x, h, eval.points, w, deriv.order=0)
   return(list(x=x, eval.points=eval.points, estimate=fhat, h=h, H=h^2, gridded=FALSE, binned=FALSE, names=NULL, w=w, deriv.order=r, deriv.ind=r))
 }
 
+
+kdde.points <- function(x, H, eval.points, w, deriv.order=0, deriv.vec=TRUE) 
+{
+  n <- nrow(x)
+  Hs <- numeric(0)
+  for (i in 1:n)
+    Hs <- rbind(Hs, H)
+  r <- deriv.order
+  fhat <- dmvnorm.deriv.mixt(x=eval.points, mus=x, Sigmas=Hs, props=w/n, deriv.order=r, deriv.vec=deriv.vec, add.index=TRUE)
+  
+  return(list(x=x, eval.points=eval.points, estimate=fhat$deriv, H=H, gridded=FALSE, binned=FALSE, names=NULL, w=w, deriv.order=r, deriv.ind=fhat$deriv.ind))
+}
+
+
+
+#############################################################################
+## Kernel functional estimation
+#############################################################################
+
+kfe.1d <- function(x, g, deriv.order, inc=1, binned=FALSE, bin.par)
+{
+  r <- deriv.order
+  psir <- dnorm.deriv.sum(x=x, sigma=g, deriv.order=r, inc=inc, binned=binned, bin.par=bin.par, kfe=TRUE)
+  if (inc==0)  psir <- psir - dnorm.deriv(0, mu=0, sigma=g, deriv.order=r)/length(x)
+  
+  return(psir) 
+}
+
+kfe <- function(x, G, deriv.order, inc=1, binned=FALSE, bin.par, bgridsize, double.loop=FALSE, deriv.vec=TRUE, add.index=TRUE, Sdr.mat, verbose=FALSE, kfold=1)
+{
+  r <- deriv.order
+  d <- ncol(x)
+  n <- nrow(x)
+
+  ##if (missing(double.loop)) double.loop <- (n*d^r/kfold > 1e6)
+  if (kfold > 1)
+  {
+    kfold.group <- sample(size=n, 1:kfold, replace=TRUE)
+    psir <- 0
+    for (k in 1:kfold)
+      psir <- psir + dmvnorm.deriv.sum(x=x[which(kfold.group==k),], Sigma=G, deriv.order=r, inc=inc, binned=FALSE, deriv.vec=deriv.vec, add.index=FALSE, double.loop=double.loop, Sdr.mat=Sdr.mat, verbose=verbose, kfe=TRUE)
+    psir <- psir/kfold
+  }
+  else
+  {  
+    psir <- dmvnorm.deriv.sum(x=x, Sigma=G, deriv.order=r, inc=inc, binned=binned, double.loop=double.loop, bin.par=bin.par, bgridsize=bgridsize, deriv.vec=deriv.vec, verbose=verbose, Sdr.mat=Sdr.mat, kfe=TRUE)
+  }
+  psir <- drop(psir)
+  
+  
+  if (add.index)
+  {
+    ind.mat <- dmvnorm.deriv(x=rep(0,d), mu=rep(0,d), Sigma=diag(d), deriv.order=r, only.index=TRUE)
+  
+    if (deriv.vec) return(list(psir=psir, deriv.ind=ind.mat))
+    else return(list(psir=psir, deriv.ind=unique(ind.mat)))
+  }
+  else return(psir=psir)
+}
+
+
+kfe.scalar <- function(x, g, deriv.order, inc=1, binned=FALSE, bin.par, double.loop=FALSE)
+{
+  r <- deriv.order
+  d <- ncol(x)
+  if (missing(bin.par) & binned) bin.par <- binning(x=x, H=g^2*diag(d))
+  
+  psir <- dmvnorm.deriv.scalar.sum(x=x, sigma=g, deriv.order=r, inc=inc, kfe=TRUE, binned=binned, double.loop=double.loop, bin.par=bin.par)
+  return(psir)
+}
+
+
 #####################################################################
-### Matt Wand's version of binned kernel density derivative estimation
-###
-### Computes the mth derivative of a binned
-### d-variate kernel density estimate based
-### on grid counts.
+## Matt Wand's version of binned kernel density derivative estimation
+## Used in the feature library
+##
+## Computes the mth derivative of a binned
+## d-variate kernel density estimate based
+## on grid counts.
 #############################################################
 
 drvkde <- function(x,drv,bandwidth,gridsize,range.x,binned=FALSE,se=TRUE, w)
@@ -463,7 +560,6 @@ drvkde <- function(x,drv,bandwidth,gridsize,range.x,binned=FALSE,se=TRUE, w)
      if (se) est.var <- ((symconv2D.ks((n*kappam)^2,gcounts)/n) - est^2)/(n-1)
    }
      
-
    if (d==3)
    {
      est <- symconv3D.ks(kappam,gcounts,skewflag=(-1)^drv) 
@@ -485,132 +581,3 @@ drvkde <- function(x,drv,bandwidth,gridsize,range.x,binned=FALSE,se=TRUE, w)
      return(list(x.grid=gpoints,est=est))
 }
 
-
-#############################################################################
-## Kernel functional estimation
-#############################################################################
-
-kfe.1d <- function(x, g, deriv.order, inc=1, binned=FALSE, bin.par)
-{
-  r <- deriv.order
-  if (!binned)
-    psir <- dnorm.deriv.sum(x=x, sigma=g, deriv.order=r, inc=inc, binned=FALSE, kfe=TRUE)
-  else
-  {
-    if (missing(bin.par)) bin.par <- binning(x, h=g)
-    psir <- bkfe(x=bin.par$counts, bandwidth=g, drv=r, binned=TRUE, range.x=range(bin.par$eval.points))
-    if (inc==0)  psir <- psir - dnorm.deriv(0,mu=0,sigma=g, deriv.order=r)/sum(bin.par$counts)
-  } 
-  
-  return(psir) 
-}
-
-kfe <- function(x, G, deriv.order, inc=1, binned=FALSE, bin.par, double.loop=FALSE, deriv.vec=TRUE, add.index=TRUE, Sdr.mat, verbose=FALSE)
-{
-  r <- deriv.order
-  d <- ncol(x)
-  if (missing(bin.par) & binned) bin.par <- binning(x=x, H=G)
-  
-  psir <- dmvnorm.deriv.sum(x=x, Sigma=G, deriv.order=r, inc=inc, kfe=TRUE, binned=binned, double.loop=double.loop, bin.par=bin.par, deriv.vec=deriv.vec, verbose=verbose)
-  
-  if (add.index)
-  {
-    ind.mat <- dmvnorm.deriv(x=rep(0,d), mu=rep(0,d), Sigma=diag(d), deriv.order=r, only.index=TRUE)
-  
-    if (deriv.vec) return(list(psir=psir, deriv.ind=ind.mat))
-    else return(list(psir=psir, deriv.ind=unique(ind.mat)))
-  }
-  else return(psir=psir)
-}
-
-
-kfe.scalar <- function(x, g, deriv.order, inc=1, binned=FALSE, bin.par, double.loop=FALSE)
-{
-  r <- deriv.order
-  d <- ncol(x)
-  if (missing(bin.par) & binned) bin.par <- binning(x=x, H=g^2*diag(d))
-  
-  psir <- dmvnorm.deriv.scalar.sum(x=x, sigma=g, deriv.order=r, inc=inc, kfe=TRUE, binned=binned, double.loop=double.loop, bin.par=bin.par)
-  return(psir)
-}
-
-
-#############################################################################
-## Estimation of vec Psi_r (no binning, arbitrary d) for unconstrained G
-## Code by Jose E. Chacon 04/09/2007
-## Has been absorbed into kfe 08/09/2010
-#############################################################################
-
-vecPsir <- function(x, Gr, Sdr, r, upper, nlim=1e4)
-{
-  d <- ncol(x)
-  n <- nrow(x)
-  S <- var(x)
-
-  ## matrix of differences - upper triangular form is available
-  difs <- differences(x, upper=upper)
-  
-  Id1 <- diag(d)
-  vId <- vec(Id1)
-  Gr12 <- matrix.sqrt(Gr)
-  Grinv12 <- chol2inv(chol(Gr12))
-
-  args <- t(Grinv12%*%t(difs))
-
-  if (r==6)
-  {
-    Kr <- function(args)
-    {
-      return(mat.Kpow(args,6)-15*mat.Kprod(mat.Kpow(args,4),rep(1,nrow(args))%x%t(vId)) + 45*mat.Kprod(mat.Kpow(args,2),rep(1,nrow(args))%x%t(Kpow(vId,2)))-15*rep(1,nrow(args))%x%t(Kpow(vId,3)))
-    }
-  }
-  else if (r==4)
-  {
-    Kr <- function(args)
-    {
-      return(mat.Kpow(args,4)-6*mat.Kprod(mat.Kpow(args,2),rep(1,nrow(args))%x%t(vId))+3*rep(1,nrow(args))%x%t(Kpow(vId,2)))
-    }
-  }
-
-  if (nrow(args) > nlim)
-  {
-    vecPsir <- rep(0, d^r)
-    num.loops <- nrow(args) %/% nlim
-    for (j in 1:num.loops)
-    {
-      args.temp <- args[((j-1)* nlim+1):(j*nlim),]
-      hmnew.temp <- Kr(args.temp)
-      vecPsir  <- vecPsir + colSums(dmvnorm(args.temp)*hmnew.temp)
-    }
-    
-    if (nrow(args) %% nlim >0)
-    {
-      args.temp <- args[(num.loops*nlim+1):nrow(args),]
-      hmnew.temp <- Kr(args.temp)
-      vecPsir  <- vecPsir + colSums(dmvnorm(args.temp)*hmnew.temp)
-    }
-  }
-  else
-  {
-    hmnew <- Kr(args)
-    vecPsir <- colSums(dmvnorm(args)*hmnew)
-  }
-
-  if (upper)
-  {
-    ## adjust for upper triangular form of differences matrix
-    args0 <- t(rep(0,d))
-    hmnew0 <- Kr(args0)
-    vecPsir <- 2*vecPsir - as.vector(n*dmvnorm(args0)*hmnew0)
-    vecPsir <- Sdr%*%vecPsir/n^2
-    }
-  else
-    vecPsir <- Sdr%*%vecPsir/nrow(args)
-  
-  vecPsir <- (-1)^r*det(Grinv12)*Kpow(Grinv12,r)%*%vecPsir
-
-  return(vecPsir)
-}
-
-
-
